#!/usr/bin/env python3

"""Module containing the GOdMDPrep class and the command line interface."""
import argparse
from typing import Optional
from pathlib import Path
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.configuration import settings
from biobb_common.tools import file_utils as fu
from biobb_common.tools.file_utils import launchlogger
from biobb_godmd.godmd.common import check_input_path, check_output_path


class GOdMDPrep(BiobbObject):
    """
    | biobb_godmd GOdMDPrep
    | Helper bb to prepare inputs for the `GOdMD tool <http://mmb.irbbarcelona.org/GOdMD/>`_ module.
    | Prepares input files for the GOdMD tool.

    Args:
        input_pdb_orig_path (str): Input PDB file to be used as origin in the conformational transition. File type: input. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/data/godmd/1ake_A.pdb>`_. Accepted formats: pdb (edam:format_1476).
        input_pdb_target_path (str): Input PDB file to be used as target in the conformational transition. File type: input. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/data/godmd/4ake_A.pdb>`_. Accepted formats: pdb (edam:format_1476).
        output_aln_orig_path (str): Output GOdMD alignment file corresponding to the origin structure of the conformational transition. File type: output. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/data/godmd/1ake_A.aln>`_. Accepted formats: aln (edam:format_2330), txt (edam:format_2330).
        output_aln_target_path (str): Output GOdMD alignment file corresponding to the target structure of the conformational transition. File type: output. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/data/godmd/4ake_A.aln>`_. Accepted formats: aln (edam:format_2330), txt (edam:format_2330).
        properties (dict - Python dictionary object containing the tool parameters, not input/output files):
            * **gapopen** (*float*) - (12.0) Standard gap penalty: score taken away when a gap is created.
            * **gapextend** (*float*) - (2.0) Penalty added to the standard gap penalty for each base or residue in the gap.
            * **datafile** (*str*) - ("EPAM250") Scoring matrix file used when comparing sequences.
            * **binary_path** (*str*) - ("water") Binary path.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_godmd.godmd.godmd_prep import godmd_prep
            prop = {
                'gapopen': 10.0,
                'gapextend': 2.0
            }
            godmd_prep( input_pdb_orig_path='/path/to/input_orig.pdb',
                        input_pdb_target_path='/path/to/input_target.pdb',
                        output_aln_orig_path='/path/to/orig.aln',
                        output_aln_target_path='/path/to/target.aln',
                        properties=prop)

    Info:
        * wrapped_software:
            * name: In house
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(self, input_pdb_orig_path: str, input_pdb_target_path: str,
                 output_aln_orig_path: str, output_aln_target_path: str,
                 properties: Optional[dict] = None, **kwargs) -> None:

        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        self.AA_TRANSLATOR = {'GLY': 'G',
                              'ALA': 'A',
                              'VAL': 'V',
                              'LEU': 'L',
                              'ILE': 'I',
                              'MET': 'M',
                              'PHE': 'F',
                              'TRP': 'W',
                              'PRO': 'P',
                              'SER': 'S',
                              'THR': 'T',
                              'CYS': 'C',
                              'CYX': 'C',
                              'CYM': 'C',
                              'TYR': 'Y',
                              'TYM': 'Y',
                              'ASN': 'N',
                              'GLN': 'Q',
                              'ASP': 'D',
                              'ASH': 'D',
                              'GLU': 'E',
                              'GLH': 'E',
                              'LYS': 'K',
                              'LYN': 'K',
                              'ARG': 'R',
                              'ARN': 'R',
                              'HIS': 'H',
                              'HIE': 'H',
                              'HID': 'H',
                              'HIP': 'H'}

        # Input/Output files
        self.io_dict = {
            'in': {'input_pdb_orig_path': input_pdb_orig_path,
                   'input_pdb_target_path': input_pdb_target_path},
            'out': {'output_aln_orig_path': output_aln_orig_path,
                    'output_aln_target_path': output_aln_target_path}
        }

        # Properties specific for BB
        self.properties = properties
        self.gapopen = properties.get('gapopen', 12.0)
        self.gapextend = properties.get('gapextend', 2.0)
        self.datafile = properties.get('datafile', "EPAM250")
        self.binary_path = properties.get('binary_path', "water")

        # Check the properties
        self.check_properties(properties)
        self.check_arguments()

    def check_data_params(self, out_log, out_err):
        """ Checks input/output paths correctness """

        # Check input(s)
        self.io_dict["in"]["input_pdb_orig_path"] = check_input_path(self.io_dict["in"]["input_pdb_orig_path"], "input_pdb_orig_path", False, out_log, self.__class__.__name__)
        self.io_dict["in"]["input_pdb_target_path"] = check_input_path(self.io_dict["in"]["input_pdb_target_path"], "input_pdb_target_path", False, out_log, self.__class__.__name__)

        # Check output(s)
        self.io_dict["out"]["output_aln_orig_path"] = check_output_path(self.io_dict["out"]["output_aln_orig_path"], "output_aln_orig_path", False, out_log, self.__class__.__name__)
        self.io_dict["out"]["output_aln_target_path"] = check_output_path(self.io_dict["out"]["output_aln_target_path"], "output_aln_target_path", False, out_log, self.__class__.__name__)

    def extract_sequence(self, pdb):
        '''
        Parses a PDB file to retrieve the sequence of a certain chain.
        '''
        seq = ''
        resids = []
        with open(pdb) as file:
            for line in file:
                line = line.rstrip('\n')
                record = line[:6].replace(" ", "")
                if record != 'ATOM':
                    continue
                atomname = line[12:16]
                atomname = atomname.replace(" ", "")
                if atomname != 'CA':
                    continue
                alternate = line[16]
                if alternate != ' ':
                    if alternate != 'A':
                        continue  # if alt loc, only A-labeled residues
                resname = line[17:20].replace(" ", "")
                if resname not in self.AA_TRANSLATOR:
                    continue
                resnum = line[22:26].replace(" ", "")
                icode = line[26:27]
                seq += self.AA_TRANSLATOR[resname]
                resids += [(' ', int(resnum), icode)]  # BioPython's standard PDB atom id (' ',resid,insertioncode) (e.g. (' ',40,'B'))

        return seq, resids

    def retrieve_alignment(self, waterFile, resids1, resids2):
        '''
        Gets the FASTA sequence of two PDB structures and a list of their residues in Biopython residue format.
        Opens a file containing a local sequence alignment (Water program of EMBOSS package) between those two PDB structures.
        Returns the sequence identity and the pairs of residues of the alignment.
        '''
        # Parsing Water alignment file
        is_alignment = 0
        is_qseq = 0
        is_sseq = 0
        qseq = ""
        sseq = ""
        ident = ""
        next_seq = 0
        qstart = ""
        sstart = ""
        with open(waterFile) as file:
            for line in file:
                line = line.rstrip("\n")
                if is_alignment == 0:
                    if not line.startswith(">>#1"):
                        continue
                    else:
                        is_alignment = 1
                else:
                    if line.startswith("; sw_ident:"):
                        ident = line[12:]
                    elif line.startswith("; al_start:"):
                        if next_seq == 0:
                            qstart = line[12:]
                        else:
                            sstart = line[12:]
                    elif line.startswith("; al_display_start:"):
                        if next_seq == 0:
                            is_qseq = 1
                        else:
                            is_sseq = 1
                    else:
                        if is_qseq == 1:
                            if not line.startswith(">"):
                                qseq = qseq + line
                            else:
                                is_qseq = 0
                                next_seq = 1
                        if is_sseq == 1:
                            if line != "":
                                sseq = sseq + line
                            else:
                                is_sseq = 0

        # Get the Sequence Identity of the alignment
        seq_id = float(ident)

        # Get the residues of the alignment.
        # -2 is applied because in the first iteration of next loop all values will always be increased by 1 (local alignment)
        # and we also need to convert the number into a python index to retrieve the proper token in the list
        resid_pairs = []
        idx1 = int(qstart)-2
        idx2 = int(sstart)-2
        for i in range(len(qseq)):  # it could be also len(sseq)
            if qseq[i] != '-':
                idx1 += 1
            if sseq[i] != '-':
                idx2 += 1
            if qseq[i] == '-' or sseq[i] == '-':
                continue
            resid_pairs += [(resids1[idx1], resids2[idx2])]
        # Check contents of the residues of the alignment
        if len(resid_pairs) == 0:  # Alignment file was empty or there was no possible matching residues between the PDBs
            fu.log('Alignment file was empty or there was no possible matching residues between the PDBs' % self.stage_io_dict["unique_dir"], self.out_log)
            return False, False
        else:
            return seq_id, resid_pairs

    @launchlogger
    def launch(self):
        """Launches the execution of the GOdMDPrep module."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        # Starting the work...
        # Parsing the input PDB files
        pdb1 = self.io_dict["in"]["input_pdb_orig_path"]
        pdb2 = self.io_dict["in"]["input_pdb_target_path"]

        # Generate sequence of first PDB
        seq1, resids1 = self.extract_sequence(pdb1)

        # Generate sequence of second PDB
        seq2, resids2 = self.extract_sequence(pdb2)

        if len(seq1) < 50:
            fu.log('WARNING: Short sequence (ORIGIN)' % self.stage_io_dict["unique_dir"], self.out_log)
        if len(seq2) < 50:
            fu.log('WARNING: Short sequence (TARGET)' % self.stage_io_dict["unique_dir"], self.out_log)

        # Produce FASTA files
        name1 = "fasta1.fa"
        name2 = "fasta2.fa"
        fasta1Filename = str(Path(self.stage_io_dict["unique_dir"]).joinpath(name1))
        fasta1File = open(fasta1Filename, "w")
        fasta1File.write(">"+pdb1+"\n"+seq1)
        fasta1File.close()
        fasta2Filename = str(Path(self.stage_io_dict["unique_dir"]).joinpath(name2))
        fasta2File = open(fasta2Filename, "w")
        fasta2File.write(">"+pdb2+"\n"+seq2)
        fasta2File.close()

        waterFilename = str(Path(self.stage_io_dict["unique_dir"]).joinpath("water_align.out"))

        # water -auto -outfile=water_align.out -asequence=1ake.chains.nolig.pdb.fa
        # -bsequence=4ake.chains.pdb.fa -gapopen=12 -gapextend=2
        # -datafile=EPAM250 -aformat=markx10

        # Command line
        self.cmd = [self.binary_path,
                    '-auto',
                    '-outfile', waterFilename,
                    '-asequence', fasta1Filename,
                    '-bsequence', fasta2Filename,
                    '-gapopen', str(self.gapopen),
                    '-gapextend', str(self.gapextend),
                    '-datafile', self.datafile,
                    '-aformat', "markx10"
                    ]

        # Run Biobb block
        self.run_biobb()

        # Starting post-alignment process: generating .aln files

        # Retrieve sequence identity of the pair, based on local sequence alignment
        # previously computed, and generate the pairs of residues from both structres
        # to drive the superimposition
        seq_id, resid_pairs = self.retrieve_alignment(waterFilename, resids1, resids2)

        aln1Filename = self.io_dict["out"]["output_aln_orig_path"]
        aln1File = open(aln1Filename, 'w')
        for pair in resid_pairs:
            aln1File.write("%s%s\n" % (pair[0][1], pair[0][2].replace(" ", "")))
        aln1File.close()

        aln2Filename = self.io_dict["out"]["output_aln_target_path"]
        aln2File = open(aln2Filename, 'w')
        for pair in resid_pairs:
            aln2File.write("%s%s\n" % (pair[1][1], pair[1][2].replace(" ", "")))
        aln2File.close()

        # Copy files to host
        self.copy_to_host()

        # remove temporary folder(s)
        # self.tmp_files.extend([
        #     self.stage_io_dict.get("unique_dir", "")
        # ])
        self.remove_tmp_files()

        self.check_arguments(output_files_created=True, raise_exception=False)

        return self.return_code


def godmd_prep(input_pdb_orig_path: str, input_pdb_target_path: str,
               output_aln_orig_path: str, output_aln_target_path: str,
               properties: Optional[dict] = None, **kwargs) -> int:
    """Create :class:`GOdMDPrep <godmd.godmd_prep.GOdMDPrep>`godmd.godmd_prep.GOdMDPrep class and
    execute :meth:`launch() <godmd.godmd_prep.GOdMDPrep.launch>` method"""

    return GOdMDPrep(input_pdb_orig_path=input_pdb_orig_path,
                     input_pdb_target_path=input_pdb_target_path,
                     output_aln_orig_path=output_aln_orig_path,
                     output_aln_target_path=output_aln_target_path,
                     properties=properties).launch()

    godmd_prep.__doc__ = GOdMDPrep.__doc__


def main():
    parser = argparse.ArgumentParser(description='Prepares input files for the GOdMD tool.', formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999))
    parser.add_argument('--config', required=False, help='Configuration file')

    # Specific args
    required_args = parser.add_argument_group('required arguments')
    required_args.add_argument('--input_pdb_orig_path', required=True, help='Input PDB file to be used as origin in the conformational transition. Accepted formats: pdb.')
    required_args.add_argument('--input_pdb_target_path', required=True, help='Input PDB file to be used as target in the conformational transition. Accepted formats: pdb.')
    required_args.add_argument('--output_aln_orig_path', required=True, help='Output GOdMD alignment file corresponding to the origin structure of the conformational transition. Accepted formats: aln, txt.')
    required_args.add_argument('--output_aln_target_path', required=True, help='Output GOdMD alignment file corresponding to the target structure of the conformational transition. Accepted formats: aln, txt.')

    args = parser.parse_args()
    # config = args.config if args.config else None
    args.config = args.config or "{}"
    # properties = settings.ConfReader(config=config).get_prop_dic()
    properties = settings.ConfReader(config=args.config).get_prop_dic()

    # Specific call
    godmd_prep(input_pdb_orig_path=args.input_pdb_orig_path,
               input_pdb_target_path=args.input_pdb_target_path,
               output_aln_orig_path=args.output_aln_orig_path,
               output_aln_target_path=args.output_aln_target_path,
               properties=properties)


if __name__ == '__main__':
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="biobb_godmd",
    version="5.1.0",
    author="Biobb developers",
    author_email="adam.hospital@irbbarcelona.org",
    description="Biobb_godmd is a BioBB category for GOdMD tool (protein conformational transitions).",
    long_description="Biobb_godmd allows the calculation of protein conformational transitions using the GOdMD tool.",
    long_description_content_type="text/markdown",
    keywords="Bioinformatics Workflows BioExcel Compatibility Ensemble Protein Transitions GOdMD",
    url="https://github.com/bioexcel/biobb_godmd",
    project_urls={
        "Documentation": "http://biobb-godmd.readthedocs.io/en/latest/",
        "Bioexcel": "https://bioexcel.eu/",
    },
    packages=setuptools.find_packages(exclude=["docs", "test"]),
    package_data={"biobb_godmd": ["py.typed"]},
    install_requires=["biobb_common==5.1.0"],
    python_requires=">=3.9,<3.11",
    entry_points={
        "console_scripts": [
            "godmd_prep = biobb_godmd.godmd.godmd_prep:main",
            "godmd_run = biobb_godmd.godmd.godmd_run:main",
        ]
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: Unix",
    ],
)



#!/usr/bin/env python3

"""Module containing the GOdMDRun class and the command line interface."""

import argparse
import shutil
from pathlib import Path, PurePath
from typing import Optional

from biobb_common.configuration import settings
from biobb_common.generic.biobb_object import BiobbObject
from biobb_common.tools.file_utils import launchlogger

from biobb_godmd.godmd.common import check_input_path, check_output_path


class GOdMDRun(BiobbObject):
    """
    | biobb_godmd GOdMDRun
    | Wrapper of the `GOdMD tool <http://mmb.irbbarcelona.org/GOdMD/>`_ module.
    | Computes conformational transition trajectories for proteins using GOdMD tool.

    Args:
        input_pdb_orig_path (str): Input PDB file to be used as origin in the conformational transition. File type: input. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/data/godmd/1ake_A.pdb>`_. Accepted formats: pdb (edam:format_1476).
        input_pdb_target_path (str): Input PDB file to be used as target in the conformational transition. File type: input. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/data/godmd/4ake_A.pdb>`_. Accepted formats: pdb (edam:format_1476).
        input_aln_orig_path (str): Input GOdMD alignment file corresponding to the origin structure of the conformational transition. File type: input. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/data/godmd/1ake_A.aln>`_. Accepted formats: aln (edam:format_2330), txt (edam:format_2330).
        input_aln_target_path (str): Input GOdMD alignment file corresponding to the target structure of the conformational transition. File type: input. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/data/godmd/4ake_A.aln>`_. Accepted formats: aln (edam:format_2330), txt (edam:format_2330).
        input_config_path (str) (Optional): Input GOdMD configuration file. File type: input. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/data/godmd/params.in>`_. Accepted formats: in (edam:format_2330), txt (edam:format_2330).
        output_log_path (str): Output log file. File type: output. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/reference/godmd/godmd.log>`_. Accepted formats: log (edam:format_2330), out (edam:format_2330), txt (edam:format_2330), o (edam:format_2330).
        output_ene_path (str): Output energy file. File type: output. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/reference/godmd/godmd_ene.out>`_. Accepted formats: log (edam:format_2330), out (edam:format_2330), txt (edam:format_2330), o (edam:format_2330).
        output_trj_path (str): Output trajectory file. File type: output. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/reference/godmd/godmd_trj.mdcrd>`_. Accepted formats: trj (edam:format_3878), crd (edam:format_3878), mdcrd (edam:format_3878), x (edam:format_3878).
        output_pdb_path (str): Output structure file. File type: output. `Sample file <https://github.com/bioexcel/biobb_godmd/raw/master/biobb_godmd/test/reference/godmd/godmd_pdb.pdb>`_. Accepted formats: pdb (edam:format_1476).
        properties (dict - Python dictionary object containing the tool parameters, not input/output files):
            * **godmdin** (*dict*) - ({}) GOdMD options specification.
            * **binary_path** (*str*) - ("discrete") Binary path.
            * **remove_tmp** (*bool*) - (True) [WF property] Remove temporal files.
            * **restart** (*bool*) - (False) [WF property] Do not execute if output files exist.
            * **sandbox_path** (*str*) - ("./") [WF property] Parent path to the sandbox directory.

    Examples:
        This is a use example of how to use the building block from Python::

            from biobb_godmd.godmd.godmd_run import godmd_run
            prop = {
                'remove_tmp': True
            }
            godmd_run(   input_pdb_orig_path='/path/to/pdb_orig.pdb',
                         input_pdb_target_path='/path/to/pdb_target.pdb',
                         input_aln_orig_path='/path/to/aln_orig.aln',
                         input_aln_target_path='/path/to/aln_target.aln',
                         output_log_path='/path/to/godmd_log.log',
                         output_ene_path='/path/to/godmd_ene.txt',
                         output_trj_path='/path/to/godmd_trj.mdcrd',
                         output_pdb_path='/path/to/godmd_pdb.pdb',
                         properties=prop)

    Info:
        * wrapped_software:
            * name: GOdMD
            * version: >=1.0
            * license: Apache-2.0
        * ontology:
            * name: EDAM
            * schema: http://edamontology.org/EDAM.owl

    """

    def __init__(
        self,
        input_pdb_orig_path: str,
        input_pdb_target_path: str,
        input_aln_orig_path: str,
        input_aln_target_path: str,
        input_config_path: Optional[str],
        output_log_path: str,
        output_ene_path: str,
        output_trj_path: str,
        output_pdb_path: str,
        properties: Optional[dict] = None,
        **kwargs,
    ) -> None:
        properties = properties or {}

        # Call parent class constructor
        super().__init__(properties)
        self.locals_var_dict = locals().copy()

        # Input/Output files
        self.io_dict = {
            "in": {
                "input_pdb_orig_path": input_pdb_orig_path,
                "input_pdb_target_path": input_pdb_target_path,
                "input_aln_orig_path": input_aln_orig_path,
                "input_aln_target_path": input_aln_target_path,
                "input_config_path": input_config_path,
            },
            "out": {
                "output_log_path": output_log_path,
                "output_ene_path": output_ene_path,
                "output_trj_path": output_trj_path,
                "output_pdb_path": output_pdb_path,
            },
        }

        # Properties specific for BB
        self.properties = properties
        self.godmdin = {k: str(v) for k, v in properties.get("godmdin", dict()).items()}
        self.binary_path = properties.get("binary_path", "discrete")

        # Check the properties
        self.check_properties(properties)
        # self.check_arguments()

    def check_data_params(self, out_log, out_err):
        """Checks input/output paths correctness"""

        # Check input(s)
        self.io_dict["in"]["input_pdb_orig_path"] = check_input_path(
            self.io_dict["in"]["input_pdb_orig_path"],
            "input_pdb_orig_path",
            False,
            out_log,
            self.__class__.__name__,
        )
        self.io_dict["in"]["input_pdb_target_path"] = check_input_path(
            self.io_dict["in"]["input_pdb_target_path"],
            "input_pdb_target_path",
            False,
            out_log,
            self.__class__.__name__,
        )
        self.io_dict["in"]["input_aln_orig_path"] = check_input_path(
            self.io_dict["in"]["input_aln_orig_path"],
            "input_aln_orig_path",
            False,
            out_log,
            self.__class__.__name__,
        )
        self.io_dict["in"]["input_aln_target_path"] = check_input_path(
            self.io_dict["in"]["input_aln_target_path"],
            "input_aln_target_path",
            False,
            out_log,
            self.__class__.__name__,
        )
        self.io_dict["in"]["input_config_path"] = check_input_path(
            self.io_dict["in"]["input_config_path"],
            "input_config_path",
            True,
            out_log,
            self.__class__.__name__,
        )

        # Check output(s)
        self.io_dict["out"]["output_log_path"] = check_output_path(
            self.io_dict["out"]["output_log_path"],
            "output_log_path",
            False,
            out_log,
            self.__class__.__name__,
        )
        self.io_dict["out"]["output_ene_path"] = check_output_path(
            self.io_dict["out"]["output_ene_path"],
            "output_ene_path",
            False,
            out_log,
            self.__class__.__name__,
        )
        self.io_dict["out"]["output_trj_path"] = check_output_path(
            self.io_dict["out"]["output_trj_path"],
            "output_trj_path",
            False,
            out_log,
            self.__class__.__name__,
        )
        self.io_dict["out"]["output_pdb_path"] = check_output_path(
            self.io_dict["out"]["output_pdb_path"],
            "output_pdb_path",
            False,
            out_log,
            self.__class__.__name__,
        )

    def create_godmdin(self, path: Optional[str] = None) -> str:
        """Creates a GOdMD configuration file (godmdin) using the properties file settings"""
        godmdin_list = []

        self.output_godmdin_path = path

        if self.io_dict["in"]["input_config_path"]:
            # GOdMD input parameters read from an input godmdin file
            with open(self.io_dict["in"]["input_config_path"]) as input_params:
                for line in input_params:
                    if "=" in line:
                        godmdin_list.append(line.upper())
        else:
            # Pre-configured simulation type parameters
            godmdin_list.append("  TSNAP = 500 ! BioBB GOdMD default params \n")
            godmdin_list.append("  TEMP = 300 ! BioBB GOdMD default params \n")
            godmdin_list.append("  SEED = 2525 ! BioBB GOdMD default params \n")
            godmdin_list.append("  ENER_EVO_SIZE = 20 ! BioBB GOdMD default params \n")
            godmdin_list.append("  NBLOC = 10000 ! BioBB GOdMD default params \n")
            godmdin_list.append(
                "  ERRORACCEPTABLE = 1.5 ! BioBB GOdMD default params \n"
            )

        # Adding the rest of parameters in the config file to the mdin file
        # if the parameter has already been added replace the value
        parameter_keys = [parameter.split("=")[0].strip() for parameter in godmdin_list]
        for k, v in self.godmdin.items():
            config_parameter_key = str(k).strip().upper()
            if config_parameter_key in parameter_keys:
                godmdin_list[parameter_keys.index(config_parameter_key)] = (
                    "\t" + config_parameter_key + " = " + str(v) + " ! BioBB property \n"
                )
            else:
                godmdin_list.append(
                    "\t" + config_parameter_key + " = " + str(v) + " ! BioBB property \n"
                )

        # Writing MD configuration file (mdin)
        with open(str(self.output_godmdin_path), "w") as godmdin:
            # GOdMDIN parameters added by the biobb_godmd module
            godmdin.write(
                "!This godmdin file has been created by the biobb_godmd module from the BioBB library \n\n"
            )

            godmdin.write("&INPUT\n")

            # MD config parameters
            for line in godmdin_list:
                godmdin.write(line)

            godmdin.write("&END\n")

        return str(self.output_godmdin_path)

    @launchlogger
    def launch(self):
        """Launches the execution of the GOdMDRun module."""

        # check input/output paths and parameters
        self.check_data_params(self.out_log, self.err_log)

        # Setup Biobb
        if self.check_restart():
            return 0
        self.stage_files()

        # Creating GOdMD input file
        self.output_godmdin_path = self.create_godmdin(
            path=str(Path(self.stage_io_dict["unique_dir"]).joinpath("godmd.in"))
        )

        # Command line
        # discrete -i $fileName.in -pdbin $pdbch1 -pdbtarg $pdbch2 -ener $fileName.ene -trj $fileName.crd -p1 $alignFile1 -p2 $alignFile2 -o $fileName.log >& $fileName.out
        self.cmd = [
            "cd",
            self.stage_io_dict["unique_dir"],
            ";",
            self.binary_path,
            "-i",
            "godmd.in",
            "-pdbin",
            PurePath(self.stage_io_dict["in"]["input_pdb_orig_path"]).name,
            "-pdbtarg",
            PurePath(self.stage_io_dict["in"]["input_pdb_target_path"]).name,
            "-p1",
            PurePath(self.stage_io_dict["in"]["input_aln_orig_path"]).name,
            "-p2",
            PurePath(self.stage_io_dict["in"]["input_aln_target_path"]).name,
            "-o",
            PurePath(self.stage_io_dict["out"]["output_log_path"]).name,
            "-ener",
            PurePath(self.stage_io_dict["out"]["output_ene_path"]).name,
            "-trj",
            PurePath(self.stage_io_dict["out"]["output_trj_path"]).name,
        ]

        # Run Biobb block
        self.run_biobb()

        # Copy outputs from temporary folder to output path
        shutil.copy2(
            str(Path(self.stage_io_dict["unique_dir"]).joinpath("reference.pdb")),
            PurePath(self.io_dict["out"]["output_pdb_path"]),
        )

        # Copy files to host
        self.copy_to_host()

        # remove temporary folder(s)
        # self.tmp_files.extend([
        #     self.stage_io_dict.get("unique_dir", ""),
        #  ])
        self.remove_tmp_files()

        self.check_arguments(output_files_created=True, raise_exception=False)

        return self.return_code


def godmd_run(
    input_pdb_orig_path: str,
    input_pdb_target_path: str,
    input_aln_orig_path: str,
    input_aln_target_path: str,
    output_log_path: str,
    output_ene_path: str,
    output_trj_path: str,
    output_pdb_path: str,
    input_config_path: Optional[str] = None,
    properties: Optional[dict] = None,
    **kwargs,
) -> int:
    """Create :class:`GOdMDRun <godmd.godmd_run.GOdMDRun>`godmd.godmd_run.GOdMDRun class and
    execute :meth:`launch() <godmd.godmd_run.GOdMDRun.launch>` method"""

    return GOdMDRun(
        input_pdb_orig_path=input_pdb_orig_path,
        input_pdb_target_path=input_pdb_target_path,
        input_aln_orig_path=input_aln_orig_path,
        input_aln_target_path=input_aln_target_path,
        input_config_path=input_config_path,
        output_log_path=output_log_path,
        output_ene_path=output_ene_path,
        output_trj_path=output_trj_path,
        output_pdb_path=output_pdb_path,
        properties=properties,
    ).launch()

    godmd_run.__doc__ = GOdMDRun.__doc__


def main():
    parser = argparse.ArgumentParser(
        description="Computing conformational transition trajectories for proteins using GOdMD tool.",
        formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=99999),
    )
    parser.add_argument("--config", required=False, help="Configuration file")

    # Specific args
    required_args = parser.add_argument_group("required arguments")
    required_args.add_argument(
        "--input_pdb_orig_path",
        required=True,
        help="Input PDB file to be used as origin in the conformational transition. Accepted formats: pdb.",
    )
    required_args.add_argument(
        "--input_pdb_target_path",
        required=True,
        help="Input PDB file to be used as target in the conformational transition. Accepted formats: pdb.",
    )
    required_args.add_argument(
        "--input_aln_orig_path",
        required=True,
        help="Input GOdMD alignment file corresponding to the origin structure of the conformational transition. Accepted formats: aln, txt.",
    )
    required_args.add_argument(
        "--input_aln_target_path",
        required=True,
        help="Input GOdMD alignment file corresponding to the target structure of the conformational transition. Accepted formats: aln, txt.",
    )
    required_args.add_argument(
        "--input_config_path",
        required=False,
        help="Input configuration file (GOdMD run options). Accepted formats: in, txt.",
    )
    required_args.add_argument(
        "--output_log_path",
        required=True,
        help="Output log file. Accepted formats: log, out, txt.",
    )
    required_args.add_argument(
        "--output_ene_path",
        required=True,
        help="Output energy file. Accepted formats: log, out, txt.",
    )
    required_args.add_argument(
        "--output_trj_path",
        required=True,
        help="Output trajectory file. Accepted formats: mdcrd.",
    )
    required_args.add_argument(
        "--output_pdb_path",
        required=True,
        help="Output structure file. Accepted formats: pdb.",
    )

    args = parser.parse_args()
    # config = args.config if args.config else None
    args.config = args.config or "{}"
    # properties = settings.ConfReader(config=config).get_prop_dic()
    properties = settings.ConfReader(config=args.config).get_prop_dic()

    # Specific call
    godmd_run(
        input_pdb_orig_path=args.input_pdb_orig_path,
        input_pdb_target_path=args.input_pdb_target_path,
        input_aln_orig_path=args.input_aln_orig_path,
        input_aln_target_path=args.input_aln_target_path,
        input_config_path=args.input_config_path,
        output_log_path=args.output_log_path,
        output_ene_path=args.output_ene_path,
        output_trj_path=args.output_trj_path,
        output_pdb_path=args.output_pdb_path,
        properties=properties,
    )


if __name__ == "__main__":
    main()
